use std::ops;

#[derive(Debug, Default, PartialEq, Eq, Clone, Copy)]
pub struct Proficiencies {
    // Armor
    pub light_armor: bool,
    pub medium_armor: bool,
    pub heavy_armor: bool,
    // Weapon
    pub simple_weapons: bool,
    pub bladed_weapons: bool,
    pub blunt_weapons: bool,
    pub pole_weapons: bool,
    pub thrown_weapons: bool,
    pub ranged_weapons: bool,
}

pub trait ToProficiencies {
    fn proficiencies(&self) -> Proficiencies;
}

impl ops::BitOr for Proficiencies {
    type Output = Proficiencies;

    fn bitor(self, rhs: Self) -> Self::Output {
        Proficiencies {
            light_armor: self.light_armor || rhs.light_armor,
            medium_armor: self.medium_armor || rhs.medium_armor,
            heavy_armor: self.heavy_armor || rhs.heavy_armor,
            simple_weapons: self.simple_weapons || rhs.simple_weapons,
            bladed_weapons: self.bladed_weapons || rhs.bladed_weapons,
            blunt_weapons: self.blunt_weapons || rhs.blunt_weapons,
            pole_weapons: self.pole_weapons || rhs.pole_weapons,
            thrown_weapons: self.thrown_weapons || rhs.thrown_weapons,
            ranged_weapons: self.ranged_weapons || rhs.ranged_weapons,
        }
    }
}

#[cfg(test)]
mod skills_tests {
    use crate::{
        class::Class,
        skills::{Proficiencies, ToProficiencies},
    };

    #[test]
    fn defaults() {
        assert_eq!(
            Proficiencies::default() | Proficiencies::default(),
            Proficiencies::default()
        );
        assert_eq!(
            Class::Brawler.proficiencies() | Proficiencies::default(),
            Class::Brawler.proficiencies()
        )
    }

    #[test]
    fn combine() {
        assert_eq!(
            Proficiencies {
                light_armor: true,
                ..Default::default()
            } | Proficiencies {
                light_armor: true,
                ..Default::default()
            },
            Proficiencies {
                light_armor: true,
                ..Default::default()
            }
        );

        assert_eq!(
            Proficiencies {
                light_armor: true,
                ..Default::default()
            } | Proficiencies {
                heavy_armor: true,
                ..Default::default()
            },
            Proficiencies {
                light_armor: true,
                heavy_armor: true,
                ..Default::default()
            }
        )
    }
}
